/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.uml;



/**
 * <h2Associations</h2>
 *
 * <pre>
 *           0..1        statechart        0..1
 * UMLClass ------------------------------------ UMLReactiveDiagram
 *           uMLClass      uMLReactiveDiagram
 * </pre>
 *
 * @author    $Author: lowende $
 * @version   $Revision: 1.7 $
 */
public abstract class UMLReactiveDiagram extends UMLTransitionDiagram
{
   /**
    * <pre>
    *           0..1        statechart        0..1
    * UMLClass ------------------------------------ UMLReactiveDiagram
    *           uMLClass      uMLReactiveDiagram
    * </pre>
    */
   private UMLClass uMLClass;


   /**
    * UMLMethod: '+ setUMLClass (value: UMLClass): Boolean'.
    *
    * @param value  The new uMLClass value
    * @return       No description provided
    */
   public boolean setUMLClass (UMLClass value)
   {
      boolean changed = false;
      if (this.uMLClass != value)
      {
         if (this.uMLClass != null)
         {
            UMLClass oldValue = this.uMLClass;
            this.uMLClass = null;
            oldValue.setUMLReactiveDiagram (null);
         }
         this.uMLClass = value;
         if (value != null)
         {
            value.setUMLReactiveDiagram (this);
         }
         changed = true;
      }
      return changed;
   }


   /**
    * UMLMethod: '+ getUMLClass (): UMLClass'.
    *
    * @return   The uMLClass value
    */
   public UMLClass getUMLClass()
   {
      return this.uMLClass;
   }


   /**
    * UMLMethod: '+ removeYou (): Void'.
    */
   public void removeYou()
   {
      super.removeYou();

      UMLClass tmpUMLClass = getUMLClass();
      if (tmpUMLClass != null)
      {
         setUMLClass (null);
      } // if
   }

}

/*
 * $Log: UMLReactiveDiagram.java,v $
 * Revision 1.7  2004/11/03 11:47:55  lowende
 * Javadoc warnings removed.
 *
 */
