/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact address:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.uml.actions;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.*;

import de.uni_paderborn.fujaba.gui.EditCommentDialog;
import de.uni_paderborn.fujaba.uml.*;


/**
 * @author    $Author: lowende $
 * @version   $Revision: 1.16.2.2 $
 */
public class EditCommentAction extends AbstractAction
{
   /**
    * Action Handler for EditComment Events. Creates an EditCommentDialog
    *
    * @param event  No description provided
    */
   public void actionPerformed (ActionEvent event)
   {
      Object source = event.getSource();
      if (source instanceof Iterator)
      {
         Iterator iter = (Iterator) source;
         if (iter.hasNext())
         {
            source = iter.next();
         }
      }

      // initialize comment-editor
      EditCommentDialog commentDialog = new EditCommentDialog (UMLProject.get().getGui().getFrame());
      if (source instanceof UMLCommentary)
      {
         UMLCommentary comment = (UMLCommentary) source;
         commentDialog.unparse (comment);
      }
      else if (source instanceof UMLIncrement)
      {
         commentDialog.unparse ( ((UMLIncrement) source).getComment());
      }
      else
      { // unsupported element has been passed to this action
         return;
      }

      // now show the editor
      commentDialog.showCentered();

      // if a new comment has been created, we have to...
      if (commentDialog.isNewUMLCommentaryCreated() && source instanceof UMLIncrement)
      {
         UMLCommentary newCommentary = commentDialog.getUMLCommentary();

         if (newCommentary != null)
         {
            // ... make shure, that revComment (parent of commentary) is set!
            newCommentary.setRevComment ((UMLIncrement) source);

            // ... show the comment
            newCommentary.setVisible (true);

            UMLProject.get().refreshDisplay();
         }
      }
   }
}

/*
 * $Log: EditCommentAction.java,v $
 * Revision 1.16.2.2  2005/08/23 08:24:47  lowende
 * Removed compile warnings.
 *
 */
